/*
	File:			DTPSample.c

	Contains:		desktop printing sample code

	Written by:		Hueii H. Hopwood
	
*/

#include "DesktopPrinting.h"

#define kFinderExtension	'fext'

// function prototypes
Boolean ThirdPartyDriverSupported();
void GetGestaltDTPInfo();
OSErr SyncDTP(const FSSpec* dtpSpec);
OSErr SetDefaultDTP(const FSSpec* dtpSpec);
OSErr GetFinderAddress(AEAddressDesc *theDesc);
OSErr SendAEToFinder(Ptr dataPtr,unsigned long dataSize);


//-----------------------------------------------------------------------------------------------------
// ThirdPartyDriverSupported:
// Check the desktop printing feature gestalt to see if third-party printer driver support is available.
//-----------------------------------------------------------------------------------------------------
Boolean ThirdPartyDriverSupported()
{
    long response;
    Boolean result = false;

    OSErr err = Gestalt(gestaltDTPFeatures, &response);
    if (err == noErr)
    {
		result = !!(response & kDTPThirdPartySupported);
	}
                 
    return(result);
} //ThirdPartyDriverSupported


//-----------------------------------------------------------------------------------------------------
// GetGestaltDTPInfo:
// Use Desktop Printer Info Gestalt to get information of all of the active desktop printers.
//-----------------------------------------------------------------------------------------------------
void GetGestaltDTPInfo()
{
	OSErr	err;
	long	gestaltResponse;

	err = Gestalt (gestaltDTPInfo, &gestaltResponse);

	if ((err == noErr) && (gestaltResponse != nil))
	{
		GestaltDTPInfoHdle infoHdle = (GestaltDTPInfoHdle)gestaltResponse;
		short numDTPs = (*infoHdle)->numDTPs;

		DTPInfo* theDTPInfoPtr;
		FSSpec* theDTPDriverSpecPtr;
		short dtpCount;
			
		Handle dtpListHdle = (*infoHdle)->theDTPList;
		Handle dtpDriverListHdle = (*infoHdle)->theDTPDriverList;

		HLock(dtpListHdle);
		theDTPInfoPtr = (DTPInfo*)*dtpListHdle;

		HLock(dtpDriverListHdle);
		theDTPDriverSpecPtr = (FSSpec*)*dtpDriverListHdle;
			
		for (dtpCount = 1; dtpCount <= numDTPs; ++dtpCount)
		{
			// do something with each of the dtps and drivers
			
			// increment to the next space in the lists
			theDTPInfoPtr++;
			theDTPDriverSpecPtr++;
		}

		HUnlock (dtpListHdle);
		HUnlock (dtpDriverListHdle);
	}
} //GetGestaltDTPInfo


//-----------------------------------------------------------------------------------------------------
// SyncDTP:
// Send an AppleEvent to the finder to notify creation of a new spool file.
//-----------------------------------------------------------------------------------------------------
OSErr SyncDTP(const FSSpec* dtpSpec)
{
	OSErr err;
	DTPAppleEventData myEvent;
	
	myEvent.dtpSignature = kDTPSignature;
	myEvent.dtpEventType = aeDTPSyncEventType;
	BlockMove((Ptr)dtpSpec, (Ptr)&myEvent.dtpSpec, sizeof(FSSpec));
	
	err = SendAEToFinder((Ptr)&myEvent, sizeof(DTPAppleEventData));
	
	return(err);
} //SyncDTP


//----------------------------------------------------------------------------------------
// SetDefaultDTP:
// Send an AppleEvent to the finder to set a dtp to be the default printer
//----------------------------------------------------------------------------------------
OSErr SetDefaultDTP(const FSSpec* dtpSpec)
{
	OSErr err;
	DTPAppleEventData myEvent;

	myEvent.dtpSignature = kDTPSignature;
	myEvent.dtpEventType = aeDTPSetDefaultEventType;
	BlockMove((Ptr)dtpSpec, (Ptr)&myEvent.dtpSpec, sizeof(FSSpec));

	err = SendAEToFinder((Ptr) &myEvent, sizeof(DTPAppleEventData));

	return err;
} //SetDefaultDTP

//-----------------------------------------------------------------------------------------------------
// GetFinderAddress:
// Create the Finder address description record we wish to send Apple Event to
//-----------------------------------------------------------------------------------------------------
OSErr GetFinderAddress(AEAddressDesc *theDesc)
{
	OSErr				result = noErr;
	ProcessInfoRec		processInfo;
	ProcessSerialNumber	serialNumber;

	serialNumber.highLongOfPSN = 0;
	serialNumber.lowLongOfPSN = kNoProcess;

	while ((result = GetNextProcess(&serialNumber)) == noErr)
	{
		processInfo.processInfoLength = sizeof(ProcessInfoRec);
		processInfo.processName 	= nil;
		processInfo.processAppSpec 	= nil;

		result = GetProcessInformation(&serialNumber, &processInfo);
		if (result == noErr)
		{
			if ((processInfo.processType == 'FNDR') && (processInfo.processSignature == 'MACS'))
			{
				result = AECreateDesc(typeProcessSerialNumber, (Ptr)&serialNumber, sizeof(ProcessSerialNumber), theDesc);
				break;		// found it, exit the while loop
			}
		}
	}

	return (result);
} //GetFinderAddress


//-----------------------------------------------------------------------------------------------------
// SendAEToFinder:
// Send an AppleEvent to the Finder which will dispatch it to the desktop printer extension.
// Note that the data is sub-typed so the Finder can decipher the contents.
//
// dataPtr	-> AppleEvent data in the direct object parameter
// dataSize	-> size of the AppleEvent data
//-----------------------------------------------------------------------------------------------------
OSErr SendAEToFinder(Ptr dataPtr,unsigned long dataSize)
{
	OSErr			err = noErr;
	AEAddressDesc	finderAddress;				// the address of finder
	AppleEvent		theEvent;
	AppleEvent		replyEvent;

	err = GetFinderAddress(&finderAddress);
	if (err == noErr)
	{
		err = AECreateAppleEvent(
			kCoreEventClass,					// this is a core event
			kFinderExtension,					// for a Finder extension
			&finderAddress,						// and we send it to the Finder
			kAutoGenerateReturnID,				// we aren't getting a return
			kAnyTransactionID,					// and we don't care about the transaction #
			&theEvent);							// and we create it right here

		if (err == noErr)
		{
			err = AEPutParamPtr(
				&theEvent,						// the event to shove into
				keyDirectObject,				// direct object keyword
				kFinderExtension,				// for the Finder extension
				dataPtr,						// here's the data!
				dataSize);						// here's how long it is!

			if (err == noErr)
			{
				err = AESend(
					&theEvent,					// the event
					&replyEvent,				// no reply event because -
					kAENoReply + 				// we don't want a reply
					 kAECanInteract + 			// the receiver can interact with user
					 kAEDontReconnect,			// and don't bother to reconnect on error
					kAENormalPriority,			// just a normal event
					kAEDefaultTimeout,			// we'll wait some reasonable amount of time
					nil, nil);					// and don't care what happens during that.

			}

			// now get rid of the AppleEvent
			(void) AEDisposeDesc(&theEvent);
		}

		// and get rid of the Finder address
		(void) AEDisposeDesc(&finderAddress);
	}

	return(err);
	
} //SendAEToFinder


//-----------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------
